import usb
import sys
import array as arr
import threading
class dso_usb:
	def __init__(self):
		self.dev = None
		busses = usb.busses()
		for bus in busses:
			devices = bus.devices
			for dev in devices:
				if dev.idVendor==0xa5a5 and dev.idProduct==0x321:
					#This is our programmer!!!
					self.dev=dev
		if self.dev==None:
			raise("I can't find the scope on the USB bus!")
		self.vlock=threading.Lock()
		self.modified=False
		self.linia=[ [], [], [], [] ]
		self.quit=False
		self.auto_trig=1
		# The assignments below just create the fields in the class instance
		# The initial values are in fact set in the init_gui function
		# in the osc_gui module
		self.ack_chn=0x03; #Acq CH1 (01,02 = CH1,CH2)
		self.smp_rate=0x02; #01,02,03,04,05 .. = 100,50,25,20,10 .. MSa/s)
		self.rd2_step=0x02; #readout2 step + 1 (02,03 ..,0b)
		self.rd1_step=0x02; #readout1 step + 1 (02,03 ..,0b)
		self.ch1_trig=0x3b; # set CH1 AC, 5 V/Div, Trig CH1
							#(00,04,08 = DC,GND,AC)
							#(01,02,20,21,22,30,31,32,33 = 10m,20m,50m,0.1,0.2,0.5,1,2,5 V/Div)
							#(00,40,80 = Trig CH1,EXT,CH2)
		self.ch2_trig=0x3b; #set CH2 AC, 5 V/Div, Trig AC
							#(00,40,80 = Trig AC,TV-V,TV-H)
		self.rd2_shift=0x01; #readout2 shift (01,02,03,04,05 .. = 0,5,10,15,20 ..)
		self.trig_slope=0x01; #set Trig Slope pos (01,02 = neg,pos)
		self.ch1_v_offset=0x66;
		self.ch2_v_offset=0x5d;
		self.trig_level=0x80;
		self.ch1_offset=0xaa;
		self.ch2_offset=0x89;
		self.ch1_gain=0x7c;
		ch2_offset=0x89;
		self.ch2_gain=0xba;
	def open(self):
		self.f=self.dev.open()
		self.f.setConfiguration(self.dev.configurations[1])                                     
		self.f.claimInterface(0)
		self.f.bulkWrite(1,"\x02")                                                           
		s=self.f.bulkRead(2,30,5000)
		print("Initialized!!!")
		if s[0] != 85:
			raise("No hardware!")
		print("OK!!!")
	def get_status_byte(self):
		self.f.bulkWrite(1,"\x04")                                                           
		s=self.f.bulkRead(2,30,5000) 
		#print "Stat:"+str(s[0])+"\n"
		return s[0]
	def send_command_byte(self,b):
		self.f.bulkWrite(1,"\x03"+chr(b))
		s=self.f.bulkRead(2,30,5000)
	def download_data(self,chn,num):
		to_get=num
		w=()
		while to_get>0:
			tmp=min(240,to_get)
			self.f.bulkWrite(1,"\x09"+chr(tmp)+chr(chn))
			s=self.f.bulkRead(2,300,5000)
			w+=s
			to_get-=tmp
		return w
	def setup_parameters(self):
		#We verify if the readout 2 does not try to reach beyond the 
		#5000 samples range!!!
		if ((self.rd2_shift-1)*5+500)*self.rd2_step > 5000:
			self.rd2_shift=int((5000/self.rd2_step-500)/5)+1
			if self.rd2_shift < 1:
				self.rd2_shift = 1
			if self.rd2_shift > 255:
				self.rd2_shift = 255
		params=chr(self.ack_chn)+\
			chr(self.smp_rate)+\
			chr(self.rd2_step)+\
			chr(self.rd1_step)+\
			chr(self.ch1_trig)+\
			chr(self.ch2_trig)+\
			chr(self.rd2_shift)+\
			chr(self.trig_slope)+\
			chr(self.ch1_v_offset)+\
			chr(self.ch2_v_offset)+\
			chr(self.trig_level)+\
			chr(self.ch1_offset)+\
			chr(self.ch1_gain)+\
			chr(self.ch2_offset)+\
			chr(self.ch2_gain)
		self.vlock.acquire()
		self.f.bulkWrite(1,"\x06"+params)
		s=self.f.bulkRead(2,30,5000)
		if s[0] != 68:
			self.vlock.release()
			raise("Setup parameters error!")
		self.vlock.release()
		return
	
	def xx_norm_acq(self):
		self.send_command_byte(0x02)
		print "wait for OSC CPU"
		while True:
			stb=self.get_status_byte()
			if stb==0x21:
				break;
			elif stb!=0x00:
				raise("unexpected status: "+hex(stb)+" should be 0x00 or 0x21")
		print "wait for trigger"
		while True:
			self.send_command_byte(0x99)
			stb=self.get_status_byte()
			if stb==0x00:
				break
			else:
				print "wait for trigger: "+hex(stb)
		print "wait for data"
		while True:
			if self.get_status_byte()==0x03:
				break
		self.download_data(2)
		res=self.fd.read(16384)
		return arr.array('B',res)
		
	def auto_acq(self,chn):
		#print "wait for OSC CPU"
		self.ack_chn=chn
		self.setup_parameters()
		self.f.bulkWrite(1,"\x08"+chr(chn)+"\x10\x10")
		s=self.f.bulkRead(2,30,5000)     
		i=1
		while True:
			self.send_command_byte(0x99)
			stb=self.get_status_byte()
			i=i+1
			if stb!=0x21:
				break
			else:
				if i%1000==0 and self.auto_trig!=0:
					self.send_command_byte(0xfe)
				#print "wait for trigger: "+hex(stb)
		#print "wait for data"
		while True:
			if self.get_status_byte()==0x03:
				break
		res=self.download_data(chn,1003)
		res=arr.array('B',res)
		#Check the "magic" bytes
		if res[1000]!=0x01 or res[1001]!=0xfe or res[1002]!=0x80:
			print "Data readout error!!!\n"+hex(res[1000])+","+hex(res[1001])+","+hex(res[1002])		
		#self.send_command_byte(0xff)
		return res
